// ignore_for_file: non_constant_identifier_names

import 'dart:async';

import 'package:galaxeus_lib/galaxeus_lib.dart';
import 'package:telegram_client/tdlib/tdlib_ffi/tdlib.dart';
import 'package:telegram_client/telegram_client.dart';
import 'package:telegram_client/telegram_client_type.dart';
import "package:telegram_client/telegram_client/scheme/scheme.dart"
    as telegram_client_scheme;

/// return original data json
class TelegramClientLib {
  Tdlib tdlib;
  TelegramBotApi telegramBotApi;

  /// return original data json
  TelegramClientLib({
    required this.telegramBotApi,
    required this.tdlib,
  });

  /// return original data json
  init() async {}
}

/// return original data json
class TelegramClientData {
  int tdlib_client_id;
  TelegramClientType telegramClientType;
  String telegram_bot_api_token_bot;

  /// return original data json
  TelegramClientData({
    required this.telegram_bot_api_token_bot,
    required this.telegramClientType,
    required this.tdlib_client_id,
  });

  /// return original data json
  static TelegramClientData tdlib({
    required int tdlib_client_id,
  }) {
    return TelegramClientData(
      telegram_bot_api_token_bot: "",
      telegramClientType: TelegramClientType.tdlib,
      tdlib_client_id: tdlib_client_id,
    );
  }

  /// return original data json
  static TelegramClientData telegramBotApi({
    required String token_bot,
  }) {
    return TelegramClientData(
      telegram_bot_api_token_bot: token_bot,
      telegramClientType: TelegramClientType.telegam_bot_api,
      tdlib_client_id: 0,
    );
  }
}

/// return original data json
class TelegramClient {
  EventEmitter eventEmitter;
  late TelegramClientLib telegram_client_lib;
  late TelegramClientData telegram_client_data;

  /// return original data json
  TelegramClient({
    required TelegramClientLib telegramClientLib,
    required TelegramClientData telegramClientData,
    required this.eventEmitter,
  }) {
    telegramClientLib.tdlib.event_emitter = eventEmitter;
    telegramClientLib.telegramBotApi.event_emitter = eventEmitter;
    telegram_client_lib = telegramClientLib;
    telegram_client_data = telegramClientData;
  }

  /// return original data json
  FutureOr<telegram_client_scheme.JsonScheme> invoke({
    required Map parameters,
    TelegramClientData? telegramClientData,
    TelegramClientLib? telegramClientLib,
    bool is_form = false,
    String? urlApi,
    String? clientType,
    bool isThrowOnError = true,
    void Function(int, int)? onUploadProgress,
    bool isVoid = false,
    Duration? delayDuration,
    Duration? invokeTimeOut,
    String? extra,
    bool? isAutoGetChat,
    bool? isInvokeThrowOnError,
    FutureOr<String> Function(int, LibTdJson libTdJson)? onGenerateExtraInvoke,
    FutureOr<Map<dynamic, dynamic>> Function(String, int, LibTdJson libTdJson)?
        onGetInvokeData,
  }) async {
    telegramClientData ??= telegram_client_data;
    telegramClientLib ??= telegram_client_lib;
    if (telegramClientData.telegramClientType ==
        TelegramClientType.telegam_bot_api) {
      Map respond = await telegramClientLib.telegramBotApi.invoke(
        parameters["@type"],
        parameters: parameters,
        is_form: is_form,
        urlApi: urlApi,
        clientType: clientType,
        isThrowOnError: isThrowOnError,
        onUploadProgress: onUploadProgress,
      );
      if (respond["@type"] == "error") {
        return telegram_client_scheme.Error(respond);
      }
      return telegram_client_scheme.JsonScheme(respond);
    }
    if ((telegramClientData.telegramClientType == TelegramClientType.tdlib)) {
      Map respond = (await telegramClientLib.tdlib.invoke(
        parameters["@type"],
        parameters: parameters,
        isVoid: isVoid,
        delayDuration: delayDuration,
        invokeTimeOut: invokeTimeOut,
        extra: extra,
        isAutoGetChat: isAutoGetChat,
        isInvokeThrowOnError: isInvokeThrowOnError,
        onGenerateExtraInvoke: onGenerateExtraInvoke,
        onGetInvokeData: onGetInvokeData,
      ));
      if (respond["@type"] == "error") {
        return telegram_client_scheme.Error(respond);
      }
      return telegram_client_scheme.JsonScheme(respond);
    }
    telegram_client_scheme.Error error = telegram_client_scheme.Error.create(
      special_extra: "error",
      code: 500,
      message: "telegram_client_type_not_found",
      description: "",
    );

    if (isThrowOnError) {
      throw error;
    }
    return error;
  }
}
